@@ -130,6 +130,7 @@ static void       gail_entry_paste_received	   (GtkClipboard *clipboard,
 /* Callbacks */
 
 static void       gail_entry_notify_insert         (GailEntry            *entry);
+static void       gail_entry_notify_delete         (GailEntry            *entry);
 static void	  _gail_entry_insert_text_cb	   (GtkEntry     	 *entry,
                                                     gchar		 *arg1,
                                                     gint		 arg2,
@@ -245,7 +246,8 @@ static void
 gail_entry_object_init (GailEntry *entry)
 {
   entry->textutil = NULL;
-  entry->signal_name = NULL;
+  entry->signal_name_insert = NULL;
+  entry->signal_name_delete = NULL;
   entry->cursor_position = 0;
   entry->selection_bound = 0;
   entry->activate_description = NULL;
@@ -320,12 +322,11 @@ gail_entry_real_notify_gtk (GObject		*obj,
     }
   else if (strcmp (pspec->name, "editable") == 0)
     {
-      GValue value = { 0, };
+      gboolean value;
 
-      g_value_init (&value, G_TYPE_BOOLEAN);
-      g_object_get_property (obj, "editable", &value);
+      g_object_get (obj, "editable", &value, NULL);
       atk_object_notify_state_change (atk_obj, ATK_STATE_EDITABLE,
-                                               g_value_get_boolean (&value));
+                                               value);
     }
   else if (strcmp (pspec->name, "visibility") == 0)
     {
@@ -395,6 +396,11 @@ gail_entry_finalize (GObject            *object)
       g_source_remove (entry->insert_idle_handler);
       entry->insert_idle_handler = 0;
     }
+  if (entry->delete_idle_handler)
+    {
+      g_source_remove (entry->delete_idle_handler);
+      entry->delete_idle_handler = 0;
+    }
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
@@ -436,7 +442,7 @@ gail_entry_ref_state_set (AtkObject *accessible)
 {
   AtkStateSet *state_set;
   GtkEntry *entry;
-  GValue value = { 0, };
+  gboolean value;
   GtkWidget *widget;
 
   state_set = ATK_OBJECT_CLASS (parent_class)->ref_state_set (accessible);
@@ -447,9 +453,8 @@ gail_entry_ref_state_set (AtkObject *accessible)
 
   entry = GTK_ENTRY (widget);
 
-  g_value_init (&value, G_TYPE_BOOLEAN);
-  g_object_get_property (G_OBJECT (entry), "editable", &value);
-  if (g_value_get_boolean (&value))
+  g_object_get (G_OBJECT (entry), "editable", &value, NULL);
+  if (value)
     atk_state_set_add_state (state_set, ATK_STATE_EDITABLE);
   atk_state_set_add_state (state_set, ATK_STATE_SINGLE_LINE);
 
@@ -1065,25 +1070,32 @@ gail_entry_paste_received (GtkClipboard *clipboard,
 static gboolean
 idle_notify_insert (gpointer data)
 {
-  GailEntry *entry = GAIL_ENTRY (data);
+  GailEntry *entry;
 
+  GDK_THREADS_ENTER ();
+
+  entry = GAIL_ENTRY (data);
   entry->insert_idle_handler = 0;
   gail_entry_notify_insert (entry);
+
+  GDK_THREADS_LEAVE ();
+
   return FALSE;
 }
 
 static void
 gail_entry_notify_insert (GailEntry *entry)
 {
-  if (entry->signal_name)
+  if (entry->signal_name_insert)
     {
       g_signal_emit_by_name (entry, 
-                             entry->signal_name,
-                             entry->position,
-                             entry->length);
-      entry->signal_name = NULL;
+                             entry->signal_name_insert,
+                             entry->position_insert,
+                             entry->length_insert);
+      entry->signal_name_insert = NULL;
     }
 }
+
 /* Note arg1 returns the character at the start of the insert.
  * arg2 returns the number of characters inserted.
  */
@@ -1099,11 +1111,11 @@ _gail_entry_insert_text_cb (GtkEntry *entry,
 
   accessible = gtk_widget_get_accessible (GTK_WIDGET (entry));
   gail_entry = GAIL_ENTRY (accessible);
-  if (!gail_entry->signal_name)
+  if (!gail_entry->signal_name_insert)
     {
-      gail_entry->signal_name = "text_changed::insert";
-      gail_entry->position = *position;
-      gail_entry->length = arg2;
+      gail_entry->signal_name_insert = "text_changed::insert";
+      gail_entry->position_insert = *position;
+      gail_entry->length_insert = arg2;
     }
   /*
    * The signal will be emitted when the cursor position is updated.
@@ -1145,6 +1157,35 @@ gail_entry_get_character_at_offset (AtkText *text,
   return unichar;
 }
 
+static gboolean
+idle_notify_delete (gpointer data)
+{
+  GailEntry *entry;
+
+  GDK_THREADS_ENTER ();
+
+  entry = GAIL_ENTRY (data);
+  entry->delete_idle_handler = 0;
+  gail_entry_notify_delete (entry);
+
+  GDK_THREADS_LEAVE ();
+
+  return FALSE;
+}
+
+static void
+gail_entry_notify_delete (GailEntry *entry)
+{
+  if (entry->signal_name_delete)
+    {
+      g_signal_emit_by_name (entry, 
+                             entry->signal_name_delete,
+                             entry->position_delete,
+                             entry->length_delete);
+      entry->signal_name_delete = NULL;
+    }
+}
+
 /* Note arg1 returns the start of the delete range, arg2 returns the
  * end of the delete range if multiple characters are deleted.	
  */
@@ -1154,8 +1195,7 @@ _gail_entry_delete_text_cb (GtkEntry *entry,
                             gint      arg2)
 {
   AtkObject *accessible;
-
-  accessible = gtk_widget_get_accessible (GTK_WIDGET (entry));
+  GailEntry *gail_entry;
 
   /*
    * Zero length text deleted so ignore
@@ -1163,7 +1203,20 @@ _gail_entry_delete_text_cb (GtkEntry *entry,
   if (arg2 - arg1 == 0)
     return;
 
-  g_signal_emit_by_name (accessible, "text_changed::delete", arg1, arg2 - arg1);
+  accessible = gtk_widget_get_accessible (GTK_WIDGET (entry));
+  gail_entry = GAIL_ENTRY (accessible);
+  if (!gail_entry->signal_name_delete)
+    {
+      gail_entry->signal_name_delete = "text_changed::delete";
+      gail_entry->position_delete = arg1;
+      gail_entry->length_delete = arg2 - arg1;
+    }
+  /*
+   * The signal will be emitted when the cursor position is updated.
+   * or in an idle handler if it not updated.
+   */
+   if (gail_entry->delete_idle_handler == 0)
+     gail_entry->delete_idle_handler = g_idle_add (idle_notify_delete, gail_entry);
 }
 
 static void
@@ -1262,19 +1315,22 @@ idle_do_action (gpointer data)
   GailEntry *entry;
   GtkWidget *widget;
 
+  GDK_THREADS_ENTER ();
+
   entry = GAIL_ENTRY (data);
   entry->action_idle_handler = 0;
   widget = GTK_ACCESSIBLE (entry)->widget;
-  if (widget == NULL)
-    /*
-     * State is defunct
-     */
-    return FALSE;
-
-  if (!GTK_WIDGET_SENSITIVE (widget) || !GTK_WIDGET_VISIBLE (widget))
-    return FALSE;
+  if (widget == NULL /* State is defunct */ ||
+      !GTK_WIDGET_SENSITIVE (widget) || !GTK_WIDGET_VISIBLE (widget))
+    {
+      GDK_THREADS_LEAVE ();
+      return FALSE;
+    }
 
   gtk_widget_activate (widget);
+
+  GDK_THREADS_LEAVE ();
+
   return FALSE;
 }
 
