@@ -186,6 +186,20 @@ gail_notebook_ref_child (AtkObject      *obj,
   return accessible;
 }
 
+static void
+gail_notebook_page_added (GtkNotebook *gtk_notebook,
+                          GtkWidget   *child,
+                          guint        page_num,
+                          gpointer     data)
+{
+  AtkObject *atk_obj;
+  GailNotebook *notebook;
+
+  atk_obj = gtk_widget_get_accessible (GTK_WIDGET (gtk_notebook));
+  notebook = GAIL_NOTEBOOK (atk_obj);
+  create_notebook_page_accessible (notebook, gtk_notebook, page_num, FALSE, NULL);
+}
+
 static void
 gail_notebook_real_initialize (AtkObject *obj,
                                gpointer  data)
@@ -212,6 +226,10 @@ gail_notebook_real_initialize (AtkObject *obj,
                     "focus",
                     G_CALLBACK (gail_notebook_focus_cb),
                     NULL);
+  g_signal_connect (gtk_notebook,
+                    "page-added",
+                    G_CALLBACK (gail_notebook_page_added),
+                    NULL);
   g_object_weak_ref (G_OBJECT(gtk_notebook),
                      (GWeakNotify) gail_notebook_destroyed,
                      obj);                     
